/*
 * SecurityQueue is a queue that will encrypt/decrypt message when send/receive messages.
 */

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.SecretKeySpec;

import com.aliyun.mns.client.CloudQueue;
import com.aliyun.mns.common.ClientException;
import com.aliyun.mns.common.ServiceException;
import com.aliyun.mns.model.Message;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;

public class SecurityQueue {
    private static Logger logger = Logger.getLogger ("SecurityQueue"); 
    private CloudQueue innerQueue = null;
    private SecretKey secretKey = null;
    private String algorithm = null;
    private SecretKey toSecretKey(byte[] sKey) throws Exception
    {
        try{
            if(this.algorithm.equals("DES"))
            {
                DESKeySpec dks = new DESKeySpec(sKey);
                SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
                return keyFactory.generateSecret(dks); 
            }
            else
            {
                return new SecretKeySpec(sKey, this.algorithm);
            }
        } catch(Exception e)
        {
            logger.error("toSecretKey fail for sKey:"
                 + sKey + " algorithm:" + this.algorithm);
            e.printStackTrace();
            throw e;
        }
    }

    public SecurityQueue(CloudQueue rawQueue, String securityKey, String algorithm) throws Exception
    {
        this.innerQueue = rawQueue;
        this.algorithm = algorithm;
        this.secretKey = this.toSecretKey(Base64.decodeBase64(securityKey));
    }

    public CloudQueue getInnerQueue()
    {
        return this.innerQueue;
    }

    public Message putMessage(Message message) throws ServiceException, ClientException {
        //Encrypt message body.
        logger.debug("send message body before encrypt is:" + message.getMessageBodyAsRawString());
        try {
            Cipher cipher = Cipher.getInstance(this.algorithm); 
            cipher.init(Cipher.ENCRYPT_MODE, this.secretKey); 
            byte [] encryptMsgBody = cipher.doFinal(message.getMessageBodyAsRawString().getBytes());
            message.setMessageBody(encryptMsgBody, Message.MessageBodyType.BASE64);
            logger.debug("message after encrypt:" + message.getMessageBodyAsRawString());
        } catch (Exception e)
        {
            logger.error("encrypt message failed!");
            e.printStackTrace();
            return null;
        }
        // send message.
        return this.innerQueue.putMessage(message);
    }

    public Message popMessage(int waitSeconds) throws ServiceException, ClientException {
        // receive message.
        Message msg = this.innerQueue.popMessage(waitSeconds);
        logger.debug("receive message body is:" + msg.getMessageBodyAsRawString());
        // decrypt message.
        try
        {
            Cipher cipher = Cipher.getInstance(this.algorithm);
            cipher.init(Cipher.DECRYPT_MODE, this.secretKey);
            msg.setBaseMessageBody(cipher.doFinal(msg.getMessageBodyAsBytes()));
            logger.debug("receive message body after decrypt:" + msg.getMessageBodyAsRawString());
        } catch (Exception e)
        {
            logger.error("Decrypt message failed!");
            e.printStackTrace();
            return null;
        }
        return msg;
    }

    public void deleteMessage(String receiptHandle) throws ServiceException, ClientException {
        this.innerQueue.deleteMessage(receiptHandle);
    }
}