/*
 * Demo for how to use SecurityQueue.
 */
import org.apache.log4j.xml.DOMConfigurator;

import com.aliyun.mns.client.CloudAccount;
import com.aliyun.mns.client.CloudQueue;
import com.aliyun.mns.client.MNSClient;
import com.aliyun.mns.common.ClientException;
import com.aliyun.mns.common.ServiceException;
import com.aliyun.mns.common.utils.ServiceSettings;
import com.aliyun.mns.model.Message;
import com.aliyun.mns.model.QueueMeta;

public class SecurityQueueDemo {

    public static void main(String[] args) {
	DOMConfigurator.configure("security_queue_demo_loggingconfig.xml");
        System.out.println("SecurityQueueDemo Start");
        // Prepare
        CloudAccount account = new CloudAccount(
                ServiceSettings.getMNSAccessKeyId(),
                ServiceSettings.getMNSAccessKeySecret(),
                ServiceSettings.getMNSAccountEndpoint());
        MNSClient client = account.getMNSClient(); //this client need only initialize once

        QueueMeta qMeta = new QueueMeta();
        qMeta.setQueueName("security-queue-test");
        CloudQueue sQueue = client.createQueue(qMeta);

        // Create security queue.
        //todo: please fill secretKey with the key generated by running SecurityKeyGenerator.java
        String secretKey = "LtlY4idl/xoxncZHhp2vcA==";  
        String algorithm = "AES";
        SecurityQueue securityQueue = null;
        try
        {
            securityQueue = new SecurityQueue(sQueue, secretKey, algorithm);
        } catch (Exception e)
        {
            System.out.println("Create security queue failed!");
        }

        try
        {
            // send message.
            Message message = new Message();
            message.setMessageBody("test raw message body", Message.MessageBodyType.RAW_STRING);
            Message msg = securityQueue.putMessage(message);
            if(msg != null)
            {
                System.out.println("Send encript message successfully, message id is:"
                    + msg.getMessageId());
            }
            else
            {
                System.out.println("Send encript message failled");
            }

            // receive message.
            Message receiveMsg = securityQueue.popMessage(30);
            if (receiveMsg != null)
            {
                System.out.println("Receive message successfully, message body is:"
                     + receiveMsg.getMessageBodyAsRawString());
            }
            else
            {
                System.out.println("Receive message failed");
            }

            // delete message.
            securityQueue.deleteMessage(receiveMsg.getReceiptHandle());
        } catch (ClientException ce){
            System.out.println("Something wrong with the network connection between client and MNS service."
                    + "Please check your network and DNS availablity.");
            ce.printStackTrace();
        } catch (ServiceException se) {
            if (se.getErrorCode().equals("TimeExpired"))
            {
                System.out.println("The request is time expired. Please check your local machine timeclock");
            }
            /*
            you can get more MNS service error code in following link.
            https://help.aliyun.com/document_detail/mns/api_reference/error_code/error_code.html?spm=5176.docmns/api_reference/error_code/error_response
            */
            se.printStackTrace();
        } catch (Exception e) {
            System.out.println("Unknown exception happened!");
            e.printStackTrace();
        } finally{
            // Delete queue and close the client.
            try{
                securityQueue.getInnerQueue().delete();
                client.close();
            }catch(Exception e)
            {}
        }
        System.out.println("SecurityQueueDemo End!");
    }
}